// =======================================================================
//  PAK 2.0 Professional Edition
//
//  My totally rewritten (first edition was dead in hdd crash) and now
//  is easy to use Westwood Studios PAK resources managing library.
//
//  Copyright (c) 1992 Westwood Studios, Inc.
//  Coded by pn0, 1994-1998.
//
//  Watcom C++ 10.0 16/32 any memory model
//
//  22 Oct 1998 - PAK 2.0 initial release
// =======================================================================


// =======================================================================
//  pak_open - opens PAK file
//
//  params:	pak - PAK structure
//		name - file name
//		mode - file opening mode
//
//  return:	 0 - success
//		-1 - error
// =======================================================================
int pak_open(PAK& pak, char* name, char* mode)
{
   if ((pak.file = fopen(name, mode)) == NULL)
      return -1;

   return 0;
}


// =======================================================================
//  pak_cache - cache PAK header in memory
//
//  params:	pak - PAK structure
//
//  return:	 0 - success
//		-1 - error seek the begining of PAK
//		-2 - error read data position
//		-3 - error allocate memory for cache
//		-4 - error return to the begin of PAK
//		-5 - error read PAK header into cache
// =======================================================================
int pak_cache(PAK& pak)
{
   if (fseek(pak.file, 0, SEEK_SET))
      return -1;

   if (fread(&pak.data_position, 4, 1, pak.file) != 1)
      return -2;

   if ((pak.header_cache = \
	(unsigned char*)malloc(pak.data_position)) == NULL)
      return -3;

   if (fseek(pak.file, 0, SEEK_SET))
      return -4;

   if (fread(pak.header_cache, pak.data_position, 1, pak.file) != 1)
      return -5;

   return 0;
}


// =======================================================================
//  pak_free - freeing cache memory
//
//  params:	pak - PAK structure
//
//  return:	void
// =======================================================================
void pak_free_cache(PAK& pak)
{
   if (pak.header_cache)
      free(pak.header_cache);
}


// =======================================================================
//  pak_close - closing PAK file
//
//  params:	pak - PAK structure
//
//  return:	 0 - success
//		>0 - error (errno contain error)
// =======================================================================
int pak_close(PAK& pak)
{
   return fclose(pak.file);
}


// =======================================================================
//  pak_find - find resource (in cache) in PAK by name
//
//  params:	pak - PAK structure
//		name - resource name
//		find_position - where to place data position if found
//
//  return:	 0 - success
//		-1 - resource not found
//		-2 - incredible error (please report!)
// =======================================================================
int pak_find(PAK& pak, char* name, unsigned long& find_position)
{
   pak.header_cache_index = 0L;
   find_position = 0L;

   while(1)
      {
        if (pak_next(pak))
           return -1;

        if (!strcmp(name, pak.header_cache_name))
           {
             find_position = pak.header_cache_data_position;
             return 0;
           };
      };

   return -2;
}


// =======================================================================
//  pak_seek - find resource (in cache) and seek to data (in file)
//
//  params:	pak - PAK structure
//		name - resource nam
//
//  return:	 0 - success
//		-1 - resource not found
//		-2 - error seek to data
// =======================================================================
int pak_seek(PAK& pak, char* name)
{
    unsigned long seek_position;

    if (pak_find(pak, name, seek_position))
       return -1;

    if (fseek(pak.file, seek_position, SEEK_SET))
       return -2;

    return 0;
}


// =======================================================================
//  pak_next - jump to next (in cache) resource
//
//  params:	pak - PAK structure
//
//  return:	 0 - success
//		-1 - cache index is out of range
//		-2 - data position in file is 0 (no more resources)
// =======================================================================
int pak_next(PAK& pak)
{
   if (pak.header_cache_index >= pak.data_position)
      return -1;

   pak.header_cache_data_position = \
      *(unsigned long*)(pak.header_cache + pak.header_cache_index);

   if (pak.header_cache_data_position == 0)
      return -2;

   pak.header_cache_index += 4;

   strcpy(pak.header_cache_name, \
	(char*)(pak.header_cache + pak.header_cache_index));

   pak.header_cache_index += (strlen(pak.header_cache_name) + 1);

   return 0;
}


// =======================================================================
//  pak_size_data - detect size of resource data
//
//  params:	pak - PAK structure
//		name - resource name
//		data_size - where to place data size if found
//
//  return:	 0 - success
//		-1 - resource not found
//		-2 - cache index is out of range
//		-3 - incredible error (please report!)
// =======================================================================
int pak_size_data(PAK& pak, char* name, unsigned long& data_size)
{
   if (pak_find(pak, name, data_size))
      return -1;

   switch (pak_next(pak))
      {
        case -1: return -2;
        case -2: data_size = filelength(pak.file->_handle) - data_size;
                 return 0;
        case  0: data_size = pak.header_cache_data_position - data_size;
                 return 0;
        default: return -3;
      }
}


// =======================================================================
//  pak_load_data - allocate memory and load resource data into
//
//  params:	pak - PAK structure
//		name - resource name
//		data_pointer - pointer to data to allocate and load into
//
//  return:	 0 - success
//		-1 - error detect resource size
//		-2 - error allocate memory
//		-3 - error read data into memory
// =======================================================================
int pak_load_data(PAK& pak, char* name, unsigned char** data_pointer)
{
   unsigned long size;

   if (pak_size_data(pak, name, size))
      return -1;

   if ((*data_pointer = (unsigned char*)malloc(size)) == NULL)
      return -2;

   if (fread(*data_pointer, size, 1, pak.file) != 1)
      return -3;

   return 0;
}


// =======================================================================
//  pak_save_data_to_file - saving resource data to standalone file
//
//  params:	name - file name
//		data_pointer - pointer to data
//		size - data size
//
//  return:	 0 - success
//		-1 - error open file
//		-2 - error write data
//		-3 - error close file
// =======================================================================
int pak_save_data_to_file(char* name, \
	unsigned char** data_pointer, unsigned long size)
{
   FILE* save_file;

   if ((save_file = fopen(name, "wb")) == NULL)
      return -1;

   if (fwrite(*data_pointer, size, 1, save_file) != 1)
      return -2;

   if (fclose(save_file))
      return -3;

   return 0;
}


// =======================================================================
//  pak_free_data - free resource memory
//
//  params:	data_pointer - pointer to data
//
//  return:	void
// =======================================================================
void pak_free_data(unsigned char** data_pointer)
{
   if (*data_pointer)
      free (*data_pointer);
}

